<?php

/*
 * Invision Power File Manager
 * Compression Library
 * Copyright (c) 2003 Invision Power Services
 * -------------------------------------
 * You may not remove the copyright or
 * redistribute the script in any form
 *
 * Modification of the Zip compression class from Zend
 * Based on :
 *
 * http://www.zend.com/codex.php?id=535&single=1
 * By Eric Mueller <eric@themepark.com?
 *
 * http://www.zend.com/codex.php?id=470&single=1
 * by Denis125 <webmaster@atlant.ru>
 *
 * Official ZIP file format: http://www.pkware.com/appnote.txt
 * Modified By: Stephen Ball <stephen@invisionfilemanager.com>
 *
 * Last Modified: 3rd March 2003 
 *
 */

class zipfile
{
    var $datasec        = array(); // array to store compressed data
    var $ctrl_dir       = array(); // central directory
    var $eof_ctrl_dir   = "\x50\x4b\x05\x06\x00\x00\x00\x00"; //end of Central directory record
    var $old_offset     = 0;
    var $slash          = NULL; // Find out the correct trialing slash for the OS
    var $root           = NULL; // The root dir
    var $cur            = NULL; // The current dir
    var $logdata        = NULL; // The create archive log

    // Adds all the files in the supplied directory
    function add_dir($name = "")
    {
        global $compress;

        // Create the log data
        $comment     = str_replace("%name", $name, $compress["folder"]);
        $comment     = str_replace("%size", size(dirsize($name)), $comment);
        $this->logdata .= $comment . "\r\n";

        if ($dp = opendir($name))
        {
            while($entry = readdir($dp))
            {
                if (is_dir($this->root . $this->cur . $entry) && $entry != "." && $entry != "..")
                {
                    // Another directory is being inserted so ensure there is a trailing slash and then call the add_dir() method
                    $entry = $name . $entry . $this->slash;
                    $this->add_dir($entry);
                }
                else if ($entry != "." && $entry != "..")
                {
                    // The entry is a single file, so call the add_file() method
                    $entry = $name . $entry;
                    $this->add_file($entry);
                }
            }
            closedir($dp); // Close the directory
        }
        else
        {
            return false;
        }
    }
    // End of the add_dir() method

    // Adds a single file to the archive
    function add_file($file)
    {
        global $compress;

        // Create the log data
        $comment        = str_replace("%name", $file, $compress["file"]);
        $comment        = str_replace("%size", size(filesize($file)), $comment);
        $this->logdata .= $comment . "\r\n";

        if (is_dir($file) == TRUE)
        {
            $this->add_dir($file . $this->slash);
        }
        else
        {
            if($fp = fopen($file, "rb"))
            {
                $file_content = fread($fp, filesize($file));
                fclose($fp);

                $this->add_data($file_content, $file);
            }
            else
            {
                return false;
            }
        }
    }
    // end of add_file() method

    // Creates the file $name and then add the content of $data to it
    function add_data($data, $name)
    {
        $name = str_replace("\\", "/", $name);

        $fr   = "\x50\x4b\x03\x04";
        $fr   .= "\x14\x00";            // ver needed to extract
        $fr   .= "\x00\x00";            // gen purpose bit flag
        $fr   .= "\x08\x00";            // compression method
        $fr   .= "\x00\x00\x00\x00";    // last mod time and date

        // "local file header" segment
        $unc_len = strlen($data);
        $crc     = crc32($data);
        $zdata   = gzcompress($data);
        $zdata   = substr(substr($zdata, 0, strlen($zdata) - 4), 2); // fix crc bug
        $c_len   = strlen($zdata);
        $fr      .= pack('V', $crc);             // crc32
        $fr      .= pack('V', $c_len);           // compressed filesize
        $fr      .= pack('V', $unc_len);         // uncompressed filesize
        $fr      .= pack('v', strlen($name));    // length of filename
        $fr      .= pack('v', 0);                // extra field length
        $fr      .= $name;

        // "file data" segment
        $fr .= $zdata;

        // "data descriptor" segment (optional but necessary if archive is not served as file)
        $fr .= pack('V', $crc);                 // crc32
        $fr .= pack('V', $c_len);               // compressed filesize
        $fr .= pack('V', $unc_len);             // uncompressed filesize

        // add this entry to array
        $this->datasec[] = $fr;
        $new_offset      = strlen(implode('', $this->datasec));

        // now add to central directory record
        $cdrec = "\x50\x4b\x01\x02";
        $cdrec .= "\x00\x00";                // version made by
        $cdrec .= "\x14\x00";                // version needed to extract
        $cdrec .= "\x00\x00";                // gen purpose bit flag
        $cdrec .= "\x08\x00";                // compression method
        $cdrec .= "\x00\x00\x00\x00";        // last mod time & date
        $cdrec .= pack('V', $crc);           // crc32
        $cdrec .= pack('V', $c_len);         // compressed filesize
        $cdrec .= pack('V', $unc_len);       // uncompressed filesize
        $cdrec .= pack('v', strlen($name));  // length of filename
        $cdrec .= pack('v', 0);              // extra field length
        $cdrec .= pack('v', 0);              // file comment length
        $cdrec .= pack('v', 0);              // disk number start
        $cdrec .= pack('v', 0);              // internal file attributes
        $cdrec .= pack('V', 32 );            // external file attributes - 'archive' bit set

        $cdrec .= pack('V', $this->old_offset); // relative offset of local header
        $this->old_offset = $new_offset;

        $cdrec .= $name;

        // optional extra field, file comment goes here save to central directory
        $this->ctrl_dir[] = $cdrec;
    }
    // End of the add_data() method


    // Dump out file
    function dump_data()
    {
        global $final_zip, $compress, $version, $license;

        // Create the log data
        $comment    = str_replace("%name", $final_zip, $compress["create"]);
        $comment    = str_replace("%date", date("l, d F Y \a\\t h:i:s A"), $comment);
        $fdata      = $comment . "\r\n\r\n";
        $fdata      .= $this->logdata;

        if ($license != 1)
        {
            // Create the log
            $comment = str_replace("%program", "Invision Power File Manager", $compress["by"]);
            $comment = str_replace("%version", $version, $comment);

            $fdata = clean_crlf($fdata . "\r\n" . $compress["done"] . "\r\n" . "\r\n" . $comment . "\r\n" . " Copyright 2003 Invision Power Services Inc (http://www.invisionpower.com)");

            $this->add_data($fdata, $comment . ".txt");
        }

        $data    = implode('', $this->datasec);
        $ctrldir = implode('', $this->ctrl_dir);

        return
            $data .
            $ctrldir .
            $this->eof_ctrl_dir .
            pack('v', sizeof($this->ctrl_dir)) .  // total # of entries "on this disk"
            pack('v', sizeof($this->ctrl_dir)) .  // total # of entries overall
            pack('V', strlen($ctrldir)) .         // size of central dir
            pack('V', strlen($data)) .            // offset to start of central dir
            "\x00\x00";                           // .zip file comment length
    }
    // End of the dump_data() method

    // Create the zip file on the disk
    function create_zip($filename)
    {
        if ($fp = fopen($filename, "wb"))
        {
            fwrite($fp, $this->dump_data());
            fclose($fp);

            return true;
        }
        else
        {
            return false;
        }
    }
    // End of create_zip() method
}
?>